function MMI_Script
% MMI_SCRIPT - 
% 
% Written by:
% -- 
% John L. Weatherwax                2005-11-12
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

[ N, D, Xs, k, Jp, Pc, Aw, mw ] = readLandSat( 'trn' );

d=2; 
[ Ys, W0 ] = initialProjection(Xs,k,d,'LDA');
%[ Ys, W0 ] = initialProjection(Xs,k,d,'rand');

% Plot two dimensional projection: 
if( 1 )
  cs = { 'yo', 'g*', 'bd', 'rs', 'bx', 'm+' }; 
  figure; 
  for i=1:length(unique(k))
    inds=find(k==i);
    plot( -Ys(inds,1), Ys(inds,2), cs{i} ); 
    if( i==1 ) hold on; end
  end
  axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on; 
  title( 'Initial Projected LandSat Data' ); 
end

% Read in random selection of 1500 data elements:
[ N, D, Xs, k, Jp, Pc ] = readLandSat( 'trn', 1500 );

method=2;
switch method,
  case 1 % stochastic gradient
    [ Ys, W, I ] = MMI_SG(Xs,k,Jp,d,W0);
  case 2 % theta parameterization
    eta = 100.0; nIter = 30; useIC = 0; 
    [ Ys, W, I ] = MMI(Xs,k,Jp,d,W0,eta,nIter,useIC);
  case 3 % matrix parameterization
    [ Ys, W, I ] = MMI_matrix_param(Xs,k,Jp,d,W0);
  otherwise
    error( 'Unknown value of method' ); 
end
save( ['mmi_script_run_d_',num2str(d),'.mat'] ); 

if( 1 )
  % Plot the results: 
  cs = { 'yo', 'g*', 'bd', 'rs', 'bx', 'm+' }; 
  figure; 
  for i=1:length(unique(k))
    inds=find(k==i);
    plot( -Ys(inds,1), Ys(inds,2), cs{i} ); 
    if( i==1 ) hold on; end
  end
  axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on;
  title( 'Final Projected LandSat Data' ); 
end

dataStr = 'LandSat'; methStr = 'MMI'; tst_OR_trn = 'trn'; 
writeLVQInput(dataStr,methStr,tst_OR_trn,Xs,Ys,k);

% Generate testing data: 
%
[ N, D, Xs, k, Jp, Pc ] = readLandSat( 'tst' );
Xs_tst = ( Aw * ( Xs.' - repmat(mw,1,size(Xs,1)) ) ).'; 
Ys_tst = Xs_tst * W; 
dataStr = 'LandSat'; methStr = 'MMI'; tst_OR_trn = 'tst'; 
writeLVQInput(dataStr,methStr,tst_OR_trn,Xs_tst,Ys_tst,k);

% Classify
%

