function [ W, I ] = MMI_SG(Xs,k,Jp,d,W0,nIter)
%
% Maximize mutual information between projected space and the class label
%
% Inputs:
%   Xs: original feature patterns of size [ D, N ]
%   k: pattern labels of size [ N, 1 ]
%   Jp: 
%   d: desired reduced dimension (must satisfy 1 <= d <= D)
%   W0: initial guess at the projection matrix of size [ D, d ]
% Outputs:
%   W: final projection matrix of size [ D, d ]
% 
% Written by:
% -- 
% John L. Weatherwax                2005-10-30
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

%MAX_ITER_NUM=20; % <- testing ...
MAX_ITER_NUM=nIter;   

MAX_OUTER_LOOPS=5; 
MAX_INNER_LOOPS=10; 

%
% Code begins:

[ N, D ] = size(Xs); 
if( d>D ) error( 'Input value of d is too large' ); end

W = W0; 
Ys = Xs * W;
sgma = halfTwoFarthestPoints( Ys ); %sgma=5;
eta_search_min=10^(-1); eta_search_max=10^10; 
no=0; 
fprintf('>no=%3d: sgma=%g\n',no,sgma);
while( 1 ) % sgma reduction/refinement loop
  ni=0; 
  while( 1 ) % mutual information reduction loop
    
    cs = { 'yo', 'g*', 'bd', 'rs', 'bx', 'm+' }; 
    figure; 
    for i=1:length(unique(k))
      inds=find(k==i);
      plot( -Ys(inds,1), +Ys(inds,2), cs{i} ); 
      if( i==1 ) hold on; end
    end
    axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on; 
    title( 'LDA Projected LandSat Data' ); 
    
    [ dICdY, dITdY, IC, IT ] = computeInformationGradients(Ys,k,sgma,Jp);
    dIdY = dITdY; I = IT; 
    Wud = dMIdW(dIdY,Xs);
    
    % Perform very naive line search to find an "optimal" eta:
    %etaRng = 10.^[ -1:8 ]; 
    etaRng = linspace( eta_search_min, eta_search_max, 10 );
    for ei=1:length(etaRng)
      Wp = W + etaRng(ei) * Wud; 
      Q = gram(Wp); Wp = Q; %clear Q; 
      Ys = Xs * Wp;
      [ VCY2, VC2Y2, VCY ] = computeInformationPotentials(Ys,k,Jp,sgma); 
      IT = VCY2 + VC2Y2 - 2*VCY;
      fprintf( '%5d: ETA=%10g; IT=%20g', ei, etaRng(ei), IT ); 
      if( ei~=length(etaRng) ) fprintf('\n'); end
      Is(ei) = IT; 
    end
    % select the best eta: 
    [dum,ii]=max(Is); fprintf('          i_opt=%d\n',ii); 
    if( ii==1 ) eta_search_min = eta_search_min/10; end
    if( ii~=length(etaRng) ) eta_search_max = etaRng(ii+1); end
    W = W + etaRng(ii) * Wud; 
    Q = gram(W); W = Q; %clear Q; 
    Ys = Xs * W;
    
    if( 0 ) 
    cs = { 'ko', 'k*', 'kd', 'ks', 'kx', 'k+' }; 
    for i=1:length(unique(k))
      inds=find(k==i);
      plot( -Ys(inds,1), Ys(inds,2), cs{i} ); 
    end
    axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on; 
    title( 'LDA Projected LandSat Data' );    
    end
    
    %pause;
    %keyboard; 
    
    if( ni==0 ) 
      deltaI = +inf; 
    else
      deltaI = I-I_old;
    end
    I_old=I; 
    fprintf( '>ni=%3d: deltaI = %g\n', ni, deltaI ); 
    
    ni=ni+1; 
    if( ni > MAX_INNER_LOOPS | deltaI < 10^(-4) ) break; end
  end
  sgma=sgma/2; 
  fprintf( '>no=%3d: sgma=%g\n', no, sgma );
  no=no+1; 
  if( no > MAX_OUTER_LOOPS | sgma < halfInClassTwoFarthestPoints( Ys, k ) ) break; end
end
