function [ Vsubcy2, Vsubc2y2, Vsubcy ] = computeInformationPotentials(Y,k,Jp,sgma)
% 
% This implements equations 34,35, and 36.
% 
% Written by:
% -- 
% John L. Weatherwax                2005-11-03
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%----- 

N = size(Y,1); 
Nc = length( unique(k) ); 

% Compute V_{(c y)^2}:
% --checked (2005-11-03)
% --checked (2005-11-12)
%
Vsubcy2 = 0; 
for pp=1:Nc
  Yp = Y( k==pp, : );
  Gtmp = bigG( Yp, Yp, 2*sgma^2 ); 
  Vsubcy2 = Vsubcy2 + sum( Gtmp(:) );
end
Vsubcy2 = Vsubcy2/(N^2); 

% Compute V_{c^2 y^2}:
% --checked (2005-11-03)
% --checked (2005-11-12)
%
Gtmp = bigG( Y, Y, 2*sgma^2 );
jSum = sum( (Jp/N).^2 ); 
Vsubc2y2 = (1/N^2)*jSum*sum( Gtmp(:) );

% Compute V_{cy}:
% --checked (2005-11-03)
% --checked (2005-11-12)
% 
Vsubcy = 0; 
for pp=1:Nc
  Yp = Y( k==pp, : ); 
  Gtmp = bigG( Yp, Y, 2*sgma^2 ); 
  Vsubcy = Vsubcy + (Jp(pp)/N)*sum( Gtmp(:) ); 
end
Vsubcy = Vsubcy/(N^2); 
