function [ N, D, X, ki, Jp, Pc, Aw, mw ] = readLandSat( tst_OR_trn, Nds )
% 
% Written by:
% -- 
% John L. Weatherwax                2005-10-30
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

rand('seed',0); randn('seed',0); 

switch lower(tst_OR_trn)
  case { 'train', 'trn' }
    thisDir=pwd; cd( fullfile( '..', 'Data', 'Landsat' ) ); load( 'sat.trn' ); tmp = sat; clear sat; cd( thisDir ); clear thisDir; 
    std_data = 1; 
  case { 'test', 'tst' }
    thisDir=pwd; cd( fullfile( '..', 'Data', 'Landsat' ) ); load( 'sat.tst' ); tmp = sat; clear sat; cd( thisDir ); clear thisDir; 
    std_data = 0; 
  otherwise
    error( 'Unknown value of tst_OR_trn' ); 
end

%
% Work with the full data set before downsampling:
% 

D = 36; % the original feature dimension
% Extract features and original class labels:
Xfull = tmp(:,1:D); k = tmp(:,D+1); clear tmp; 

keyboard; 

% Standardize the data (only if training): 
if( std_data ) 
  %S = standardize( Xfull ); 
  [ S, Aw, mw ] = Whitening_transform( Xfull.' ); S = S.'; 
  Xfull = S; clear S; 
end
N = size(Xfull,1); 

% Map original class labels to 1,2,...,Nc:
uk = unique( k );
Nc = length( uk );
ki = zeros(size(k));
Jp = zeros(1,Nc);       % The number of samples from class Jp(ii)
for ii=1:Nc
  inds = find(k==uk(ii));
  ki(inds) = ii; 
  Jp(ii) = length(inds); 
end

% Compute prior probabilities:
Pc = Jp/N;

% Order the data so that we have all class 1 instances followed by
%                                all class 2 instances followed by
%                                all class 3 ... etc.
ki2 = []; Xfull2 = []; 
for ii=1:Nc,
  inds = find(ki==ii);
  ki2 = [ ki2, repmat(ii,1,Jp(ii)) ];
  Xfull2 = [ Xfull2; Xfull(inds,:) ]; 
end; clear Xfull; 

ki = ki2; 
X = Xfull2; 

% With no argument return all the data
if( nargin<2 ) 
  
  % Trim down the number of features to a much smaller set:
% $$$   D = 5; 
% $$$   X = Xfull2( :, 1:D ); 
  
% Trim down the number of classes (keep class 3 & 6):
%
% $$$ disp( 'Selecting only two classes' ); 
% $$$ ind3 = find(ki==3); ind6 = find(ki==6); 
% $$$ X = [ X(ind3,:); X(ind6,:) ];
% $$$ ki = [ repmat(1,1,length(ind3)), repmat(2,1,length(ind6)) ];
% $$$ Jp(1:2) = []; Jp(2:end-1) = []; 
% $$$ Pc(1:2) = []; Pc(2:end-1) = []; Pc = Pc/sum(Pc);

  return; 
end

%
% Randomly downselect the data (in proportion to the priors):
%

N = Nds; 

ki2 = []; Xfull2 = []; 
for ii=1:Nc,
  if( ii ~= Nc ) 
    Jp2(ii) = round(Pc(ii)*Nds);
  else
    Jp2(ii) = Nds - sum(Jp2); 
  end
  indsOrig = find( ki==ii ); 
  indsNewAll = shuffle( indsOrig ); 
  indsNew = indsNewAll( 1:Jp2(ii) ); 
  ki2 = [ ki2, repmat(ii,1,Jp2(ii)) ]; 
  Xfull2 = [ Xfull2; X(indsNew,:) ];
end

ki = ki2; 
X = Xfull2;
Jp = Jp2; 
Pc = Jp/Nds; 


% Trim down the number of features to a much smaller set;
% $$$ D = 5; 
% $$$ X = Xfull2( :, 1:D ); 

% Trim down the number of classes (keep class 3 & 6):
%
% $$$ disp( 'Selecting only two classes' ); 
% $$$ ind3 = find(ki==3); ind6 = find(ki==6); 
% $$$ X = [ X(ind3,:); X(ind6,:) ];
% $$$ ki = [ repmat(1,1,length(ind3)), repmat(2,1,length(ind6)) ];
% $$$ Jp(1:2) = []; Jp(2:end-1) = []; 
% $$$ Pc(1:2) = []; Pc(2:end-1) = []; Pc = Pc/sum(Pc);
